## Authors
- Rolv-Arild Braaten
- August Indal
- Joakim Sæther

# User manual

This is a short manual for getting prototypes up and running

## Overview

The prototypes consist of four different packages: 
- `util` - useful methods and classes.
- `segmentation` - all the PSS models.
- `imageclassification` - the ResNet model.
- `textclassification` - the text classifier.
- `tests` - tests/examples of usages.

## Requirements
Each of the packages have different requirements, but they all require Python 3.x (tested with 3.6).
Further requirements are:


- `util`:
    - [Numpy](http://www.numpy.org/)
    - [Pandas](https://pandas.pydata.org/) (reading from csv)
    - [scikit-image](https://scikit-image.org/) (image loading and processing)
    - [Tensorflow](https://www.tensorflow.org/) (for text preprocessing)
- `segmentation`: 
    - The `util` package
    - [Numpy](http://www.numpy.org/)
    - [Gensim](https://radimrehurek.com/gensim/) (for gensim model)
    - [imagesize](https://github.com/shibukawa/imagesize_py) (for image size model)
- `imageclassification`:
    - [Tensorflow](https://www.tensorflow.org/) (preferably 2.0+ with GPU for better performance)
- `textclassification`:
    - [Pandas](https://pandas.pydata.org/) 
    - [fastai](https://www.fast.ai/)
- `tests`:
    - All the other packages
    
## Usage
To use any of the methods or classes in any of the packages, simply import them and use them.
Here is an outline of what each package includes.

- `util`:
    - contains the Page class as well as methods for confusion matrices, reading images, loading datasets and preprocessing text.
- `segmentation`:
    - contains all the PSS models: classifier, gensim, imagesize, ngram (with an optional argument to perform preprocessing on the text) and pagenumber, as well as a PSS superclass (segmentationmodel), and a combined model (multimodel).
- `imageclassification`:
    - contains a method for retrieving an image classifier
- `textclassification`:
    -  contains example for creating a textclassification model.
- `tests`:
    - examples of usages for most methods from 
