from segmentation.segmentationmodel import SegmentationModel
from util.page import Page


def euclidean(v1, v2):
    return sum((p - q) ** 2 for p, q in zip(v1, v2)) ** .5


class ClassifierModel(SegmentationModel):
    """
    Uses classifier predictions to find similarity between pages.
    """
    def __init__(self, n: int = 1):
        """
        :param n: represent the relative scale of distances. The predict method normalizes with n / (n + dist).
        """
        self.n = n

    # Overrides
    def predict(self, page1: Page, page2: Page) -> float:
        mp1 = page1.multipred
        mp2 = page2.multipred
        if mp1 is None or mp2 is None:
            return 0.0
        return self.n / (self.n + euclidean(page1.multipred, page2.multipred))
