from math import sqrt

import imagesize as imagesize

from segmentation.segmentationmodel import SegmentationModel
from util.page import Page


class ImageSizeModel(SegmentationModel):
    """
    Compares two pages by the sizes of their images.
    """

    def __init__(self, n=1):
        """
        :param n: represent the relative scale of images. The predict method normalizes with n / (n + dist).
        """
        self.n = n

    # Overrides
    def predict(self, page1: Page, page2: Page) -> float:
        if page1.image is None:
            w1, h1 = imagesize.get(filepath=page1.path)[0:2]
        else:
            h1, w1 = page1.image.shape[0:2]
        if page2.image is None:
            w2, h2 = imagesize.get(filepath=page2.path)[0:2]
        else:
            h2, w2 = page2.image.shape[0:2]

        dist = sqrt((h2 - h1) ** 2 + (w2 - w1) ** 2)
        return self.n / (self.n + dist)
