import numpy
import tensorflow
from tensorflow.python.keras.callbacks import TensorBoard, ModelCheckpoint, EarlyStopping, ReduceLROnPlateau
from tensorflow.python.keras.backend import set_floatx

from imageclassification.resnet import get_new_model
from util.datasets import txt_to_pages, get_dataset_classify
# Dataset directory
from util.image import save_image

loc = "C:/Users/rolv_/Documents/RVL-CDIP/"
# loc = "G:/RVL-CDIP/"

train = txt_to_pages(loc + "labels/train.txt", loc + "images/%s")
val = txt_to_pages(loc + "labels/val.txt", loc + "images/%s")
test = txt_to_pages(loc + "labels/test.txt", loc + "images/%s")

classes = {str(i): i for i in range(16)}

# set_floatx('float16')

cont = True
for d in [train, val, test]:
    # print(len(d))
    for p in d:
        p.path = p.path.replace(".tif", ".png")
        # if cont and p.path != "C:/Users/rolv_/Documents/RVL-CDIP/images/imagesr/r/a/g/rag90f00/0011843414.tif":
        #     continue
        # else:
        #     cont = False
        # print(p.path)
        # try:
        #     img = p.get_image(save=False)
        #     save_image(p.path.replace(".tif", ".png"), img)
        # except IndexError:
        #     print("IndexError")
        #     img = numpy.ones([1000, 1000])
        #     save_image(p.path.replace(".tif", ".png"), img)

# exit(0)

batch = 8
epochs = 32

pool = 'avg'
weights = 'imagenet'
h, w = (512, 362)
dense = None

v = 1

tb_callback = TensorBoard(log_dir=f"../res/TB-Logdir", write_images=False,
                          histogram_freq=0, write_graph=False, profile_batch=0)

es_callback = EarlyStopping(restore_best_weights=True, patience=3, verbose=v)

# lr_callback = LearningRateScheduler(lambda x, y: y * 0.7, verbose=v)

lr_callback = ReduceLROnPlateau(factor=0.5, patience=2, verbose=v)

cp_callback = ModelCheckpoint(
    f'../res/resnet-models/ResNet-rvlcdip-{(h, w)}-{dense}-{pool}.hdf5', save_best_only=True,
    save_weights_only=True, verbose=v)

model = get_new_model(len(classes), h, w,
                      dense=dense,
                      pooling=pool,
                      weights=weights,
                      lr=1e-5)

train_data = get_dataset_classify(train, classes, batch, h, w, 3, epochs=epochs, shuffle=1000)
val_data = get_dataset_classify(val, classes, batch, h, w, 3, shuffle=1000)
test_data = get_dataset_classify(test, classes, batch, h, w, 3, shuffle=1000)

model.fit(train_data,
          steps_per_epoch=len(train) // batch,
          validation_data=val_data,
          # validation_steps=len(test) // batch,
          callbacks=[tb_callback, cp_callback, es_callback, lr_callback],
          epochs=epochs,
          verbose=0,
          use_multiprocessing=True,
          workers=16)

loss, acc = model.evaluate(test_data,
                           verbose=0)

print(f"Size=({h},{w}), dense={dense}, weights={weights}, pool={pool}: loss={loss}, acc={acc}")
