import random
import time

import numpy
import tensorflow as tf
from sklearn.metrics import confusion_matrix, classification_report
from tensorflow.python.keras import Model
from tensorflow.python.keras.callbacks import TensorBoard, ModelCheckpoint, ReduceLROnPlateau, LearningRateScheduler, \
    EarlyStopping, Callback
from tensorflow.python.keras.layers import Dense
from tensorflow.python.keras.losses import categorical_crossentropy
from tensorflow.python.keras.metrics import categorical_accuracy
from tensorflow.python.keras.optimizers import Adam, adam

from imageclassification.resnet import get_saved_model, get_new_model
from util.datasets import get_pages, get_dataset_classify

config = tf.ConfigProto()
config.gpu_options.allow_growth = True
session = tf.Session(config=config)

h = 512
w = 362
d = None

name = "KleppL2"
csv_path = "../res/Klepp100k_cleaned.csv"
image_path = "D:/braArkivUttrekk/Klepp100k/%s.png"

pages = get_pages(csv_path, image_path)
classes = {v: k for k, v in dict(enumerate(sorted({p.true_class.strip().lower() for p in pages}))).items()}

cut = 4 * len(pages) // 5
train = pages[:cut]
test = pages[cut:]

cut2 = 4 * len(train) // 5
val = train[cut2:]
train = train[:cut2]

print(len(train), len(val), len(test))
print(classes)

model = get_new_model(16, h, w, dense=d, lr=None)
model = get_saved_model(f"../res/resnet-models/ResNet-rvlcdip-({h}, {w})-None-avg.hdf5", model)

# Replace output layer to get correct classes
dense = Dense(len(classes), activation='softmax')(model.layers[-2].output)
model = Model(inputs=model.input, outputs=dense)

for layer in model.layers[:-1]:
    layer.trainable = False

model.summary()

batch = 1
epochs = 1

model.compile(adam(lr=1e-5), categorical_crossentropy, [categorical_accuracy])

train_data = get_dataset_classify(train, classes, batch, h, w, 3, epochs=epochs, shuffle=1000)
val_data = get_dataset_classify(val, classes, batch, h, w, 3, epochs=epochs, shuffle=1000)

v = 1

tb_callback = TensorBoard(
    log_dir=f"C:/Users/student/TB-Logdir/{round(time.time())}",
    write_images=False,
    write_graph=False,
    histogram_freq=0)

cp_callback = ModelCheckpoint(
    filepath=f'../res/resnet-models/ResNet-{name}-({h},{w})-avg-{d}-FineTuned.hdf5',
    save_best_only=True,
    save_weights_only=True,
    verbose=v)

es_callback = EarlyStopping(
    patience=3,
    verbose=v)

lr_callback = ReduceLROnPlateau(factor=0.5, patience=2, verbose=v)

model.fit(train_data,
          steps_per_epoch=len(train) // batch,
          validation_data=val_data,
          validation_steps=len(val) // batch,
          callbacks=[tb_callback, lr_callback, cp_callback, es_callback],
          epochs=epochs,
          verbose=0)

batch = 1
epochs = 32

train_data = get_dataset_classify(train, classes, batch, h, w, 3, epochs=epochs, shuffle=1000)
val_data = get_dataset_classify(val, classes, batch, h, w, 3, epochs=epochs, shuffle=1000)
test_data = get_dataset_classify(test, classes, batch, h, w, 3)

model = get_new_model(len(classes), h, w, dense=d, lr=None)
model = get_saved_model(f'../res/resnet-models/ResNet-{name}-({h},{w})-avg-{d}-FineTuned.hdf5', model)

for layer in model.layers:
    layer.trainable = True

model.compile(adam(lr=1e-5), categorical_crossentropy, [categorical_accuracy])

cp_callback = ModelCheckpoint(
    filepath=f'../res/resnet-models/ResNet-{name}-({h},{w})-avg-{d}-general.hdf5',
    save_best_only=True,
    save_weights_only=True,
    verbose=v)

model.fit(train_data,
          steps_per_epoch=len(train) // batch,
          validation_data=val_data,
          validation_steps=len(val) // batch,
          callbacks=[tb_callback, lr_callback, cp_callback, es_callback],
          epochs=epochs,
          verbose=0)

loss, acc = model.evaluate(test_data,
                           steps=len(test) // batch,
                           verbose=0)

pred = model.predict(test_data, verbose=1, steps=len(test) // batch)

numpy.save(f"preds-{name}.npy", pred)
pred = numpy.load(f"preds-{name}.npy")

print(confusion_matrix(numpy.array([classes[p.true_class.strip().lower()] for p in test]), numpy.argmax(pred, -1), labels=list(range(len(classes)))))
print(classification_report(numpy.array([classes[p.true_class.strip().lower()] for p in test]), numpy.argmax(pred, -1)))

print(f"Loss: {loss}, acc: {acc}")
