from segmentation.multimodel import MultiModel
from util.datasets import get_pages

"""
    This example uses the proposed model in ../segmentation/multimodel.py
    To use this, create a folder containing all the images and a csv-file containing the following columns:
      - "Path" = the 32 hexadecimal code for the page.
      - "Predicted" = the predicted class from the classification model.
      - "Multipred" = the metadata with all the probabilities from the classification model.
      - "Actual" = the actual class of the page. The models does currently not support .csv-files without this column. 
        This needs to be implemented in order to use the models for real-life applications.
      - "Parent_ID" = The ID of the mother document. This is used to avoid connecting pages from different mother 
        documents.
      - "Page" = The page number of the page in its mother document.
      - "Text" = The OCR-scanned text.
    The .csv file needs to contain these headers in that order for the read_csv-method and the pages-class to work 
    properly.
"""
model = MultiModel()

path_to_csv = ""  # Insert relative or absolute path to the .csv file here.
path_to_images = ""  # Insert relative or absolute path to the folder containing the scanned images here.

image_classes = ["foto", "kart", "tegning"]  # The available classes for image prediction.
text_classes = ["vedtak", "korrespondanse", "søknad", ""]  # The available classes for text prediction.
pages = get_pages(path_to_csv, path_to_images, split_by_mother=True,
                  multipred_classes=(image_classes, text_classes))


cut = 1 * len(pages) // 10  # Change to the correct training/testing ratio. Currently 10% training and 90% testing.
x_train = pages[:cut]
x_test = pages[cut:]

model.train(x_train)

# Storing the prediction of the model in the prediction variable.
prediction, answer = model.predict_batch(x_test)

"""
    The prediction will be stored as an array of 0s and 1s where 1 represents a connection to the next page.
    The performance of the model can be found using the class in ../util/confusion.py.
"""