import numpy
import numpy as np
import tensorflow as tf
from skimage.io import imread, imsave
from skimage.measure import block_reduce
from skimage.transform import resize
from tensorflow.python import py_func, read_file
from tensorflow.python.ops.gen_image_ops import decode_jpeg
from tensorflow.python.ops.image_ops_impl import resize_images, decode_image


def get_image(file_path: str, desired_size: tuple = None, rgb_scale=255) -> np.ndarray:
    image = imread(file_path) / rgb_scale
    if desired_size is None:
        return image
    else:
        return resize_image(image, desired_size)


def get_images(file_paths: list, desired_size: tuple = None) -> np.ndarray:
    return np.array([get_image(f, desired_size) for f in file_paths])


def save_image(file_path: str, image: np.ndarray):
    imsave(file_path, image)


def resize_image(image: np.ndarray, new_size: tuple) -> np.ndarray:
    return resize(image, new_size)


def pool(image, pool_size=(24, 24, 1), method=np.min):
    return block_reduce(image, pool_size, method)


def process_image(path, h, w, c):
    def tif():
        return py_func(decode_tiff, [path, h, w, c], numpy.float32)

    def not_tif():
        file = read_file(path)
        image = decode_jpeg(file, channels=3)
        image = resize_images(image, (h, w))
        image /= 255.0  # normalize to [0,1] range
        return image

    img = tf.cond(tf.strings.regex_full_match(path, ".+tif"), tif, not_tif)
    img.set_shape([h, w, c])
    return img


def decode_tiff(path, h, w, c):
    try:
        img = get_image(str(path, 'utf-8'), desired_size=(h, w))
        img = numpy.stack((img,) * c, axis=-1)
    except ValueError:
        RuntimeWarning(f"File not found: {str(path, 'utf-8')}")
        img = numpy.ones([h, w, c])  # white image
    return img.astype(numpy.float32)
