from util import image
from util.image import *


class Page:
    def __init__(self, index, parent_id, path, pred_class, true_class, page_nr, text, multipred=None):
        self.index = int(index)
        self.parent = int(parent_id)
        self.path = path
        self.pred_class = pred_class
        self.true_class = true_class
        self.page_nr = int(page_nr)
        if text == "NULL":
            self.text = ""
        else:
            self.text = text
        self.multipred = multipred
        self.image = None

    def get_image(self, desired_size=None, save: bool = True, rgb_scale=255) -> np.ndarray:
        if self.image is None:
            img = image.get_image(self.path, desired_size, rgb_scale)
            if save:  # Save decides whether to save to variable to reduce latency or not to reduce memory usage
                self.image = img
            return img
        else:
            return self.image

    def belongs_with(self, page):
        if self.true_class is None or page.true_class is None:
            return None
        return self.true_class == page.true_class and self.parent == page.parent and self.page_nr < page.page_nr

    def __str__(self) -> str:
        return "Page[index=%s,parent=%s,path=%s,pred=%s,class=%s,page_nr=%s]" % (
            self.index, self.parent, self.path, self.pred_class, self.true_class, self.page_nr)

    def __lt__(self, other):
        return self.index < other.index
